----------------------------------------------------------------------------------
-- Company:             www.kampis-elektroecke.de
-- Engineer:            Daniel Kampert
-- 
-- Create Date:         30.01.2018 16:11:55
-- Design Name: 
-- Module Name:         I2S_Top_TB - I2S_Top_TB_Arch
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description:         Testbench for I2S transmitter
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity I2S_Top_TB is
end I2S_Top_TB;

architecture I2S_Top_TB_Arch of I2S_Top_TB is
    
    -- Settings 1
    -- Clock 12.2880 MHz
    -- 48 kHz LRCLK
    -- Clock / LRCLK Ratio = 256
    constant CLK_PERIODE : TIME := 81.3802 ns;
    constant MCLK_PRESCALER : INTEGER := 256;
    constant DATA_WIDTH_PER_CHANNEL : INTEGER := 16;

    -- Settings 2
    -- Clock 8.1920 MHz
    -- 128 kHz LRCLK
    -- Clock / LRCLK Ratio = 64
    -- constant CLK_PERIODE : TIME := 122.0703 ns;
    -- constant MCLK_PRESCALER : INTEGER := 64;

    component I2S_Top is
        Generic ( DATA_WIDTH_PER_CHANNEL : INTEGER := 16;
                  MCLK_PRESCALER : INTEGER := 256 
                  );
        Port (  Clock    : in STD_LOGIC;
                Reset    : in STD_LOGIC;
                I2S_SCLK : out STD_LOGIC;
                LRCLK    : out STD_LOGIC;
                I2S_Data : out STD_LOGIC;
                MCLK     : out STD_LOGIC
                );
    end component;
    
    signal Clock        : STD_LOGIC := '0';
    signal MCLK         : STD_LOGIC := '0';
    
    signal Reset        : STD_LOGIC := '0';
    signal I2S_SCLK     : STD_LOGIC := '0';
    signal LRCLK        : STD_LOGIC := '0';
    signal I2S_Data     : STD_LOGIC := '0';
    signal Data         : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

begin

    DUT : I2S_Top generic map ( DATA_WIDTH_PER_CHANNEL => DATA_WIDTH_PER_CHANNEL,
                                MCLK_PRESCALER => MCLK_PRESCALER
                                )
                  port map (    Clock => Clock,
                                Reset => Reset,
                                MCLK => MCLK,
                                I2S_SCLK => I2S_SCLK,
                                LRCLK => LRCLK,
                                I2S_Data => I2S_Data
                                );

    Clock_Generation : process
    begin
        wait for CLK_PERIODE / 2;
        Clock <= not Clock;
    end process;

    Stimulus : process
    begin
        -- Reset device
        Reset <= '1';
        wait for 12 us;
        Reset <= '0';
        
        -- Wait for stimulus end
        wait for 100 ms; 
    end process;
end I2S_Top_TB_Arch;