----------------------------------------------------------------------------------
-- Company:             www.kampis-elektroecke.de
-- Engineer:            Daniel Kampert 
-- 
-- Create Date:         07.02.2018 19:05:30
-- Design Name: 
-- Module Name:         I2S_Transmitter_TB - I2S_Transmitter_TB_Arch
-- Project Name: 
-- Target Devices: 
-- Tool Versions:       Vivado 2016.4
-- Description:         Testbench for I2S-Transmitter
-- 
-- Dependencies: 
-- 
-- Revision:
--  Revision 0.01   07.02.2018      File Created
--
-- Additional Comments:
--      Strobe must be high at least for 2 i2s clock cycles
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity I2S_Transmitter_TB is
end I2S_Transmitter_TB;

architecture I2S_Transmitter_TB_Arch of I2S_Transmitter_TB is

    -- Clock (50 Mhz)
    constant CLK_PERIODE : TIME := 20 ns;
    
    -- Define the package size for data signals
    constant DATA_WIDTH_PER_CHANNEL : INTEGER := 8;
    
    -- Clock prescaler for MCLK
    constant MCLK_PRESCALER : INTEGER := 2;

    component I2S_Transmitter is
        generic (   DATA_WIDTH_PER_CHANNEL : INTEGER := 32;
                    MCLK_PRESCALER : INTEGER := 8
                    );
        port (  Clock       :   in STD_LOGIC;
                Reset       :   in STD_LOGIC;
                Data        :   in STD_LOGIC_VECTOR(((DATA_WIDTH_PER_CHANNEL * 2) - 1) downto 0);
                Strobe      :   in STD_LOGIC;
                Busy        :   out STD_LOGIC;
                MCLK        :   out STD_LOGIC;
                I2S_SCLK    :   out STD_LOGIC;
                LRCLK       :   out STD_LOGIC;
                I2S_Data    :   out STD_LOGIC
                );
    end component;
        
    signal Reset        : STD_LOGIC := '0';
    signal Clock        : STD_LOGIC := '0';
    
    -- I2S signals
    signal I2S_SCLK     : STD_LOGIC := '0';
    signal I2S_Data     : STD_LOGIC := '0';
    signal LRCLK        : STD_LOGIC := '0';
    signal MCLK         : STD_LOGIC := '0';
    
    -- Handshake signals
    signal Strobe       : STD_LOGIC := '0';
    signal Busy         : STD_LOGIC := '0';
    
    signal Data         : STD_LOGIC_VECTOR(((2 * DATA_WIDTH_PER_CHANNEL) - 1) downto 0) := (others => '0');

begin

    DUT : I2S_Transmitter generic map ( DATA_WIDTH_PER_CHANNEL => DATA_WIDTH_PER_CHANNEL,
                                        MCLK_PRESCALER => MCLK_PRESCALER
                                        )
                  port map (    Clock => Clock,
                                Reset => Reset,
                                Strobe => Strobe,
                                Busy => Busy,
                                Data => Data,
                                LRCLK => LRCLK,
                                MCLK => MCLK,
                                I2S_Data => I2S_Data,
                                I2S_SCLK => I2S_SCLK
                                );

    -- Create the clock for the device
    Clock_Generation : process
    begin
        wait for CLK_PERIODE / 2;
        Clock <= not Clock;
    end process;

    -- Simulate some data transmission
    Stimulus : process
    begin
        -- Reset device
        Reset <= '1';
        wait for 10 us;
        Reset <= '0';

        -- Send data
        -- First sequence
        Data <= x"C33C";
        wait for CLK_PERIODE;
        Strobe <= '1';
        wait for CLK_PERIODE * 2 * MCLK_PRESCALER;
        Strobe <= '0';

        -- Wait for transmission end
        wait until (Busy = '0');

        -- Second sequence
        Data <= x"FF00";
        wait for 2 * CLK_PERIODE;
        Strobe <= '1';
        wait for CLK_PERIODE * 2 * MCLK_PRESCALER;
        Strobe <= '0';

        -- Wait for transmission end
        wait until (Busy = '0');  

        -- Third sequence
        Data <= x"00FF";
        wait for 2 * CLK_PERIODE;
        Strobe <= '1';
        wait for CLK_PERIODE * 2 * MCLK_PRESCALER;
        Strobe <= '0';

        -- Wait for transmission end
        wait until (Busy = '0');  
        
        -- Fourth sequence
        Data <= x"AA55";
        wait for 2 * CLK_PERIODE;
        Strobe <= '1';
        wait for CLK_PERIODE * 2 * MCLK_PRESCALER;
        Strobe <= '0';

        -- Wait for transmission end
        wait until (Busy = '0');  

        -- Fifth sequence
        Data <= x"0F06";
        wait for 2 * CLK_PERIODE;
        Strobe <= '1';
        wait for CLK_PERIODE * 2 * MCLK_PRESCALER;
        Strobe <= '0';

        -- Wait for transmission end
        wait until (Busy = '0'); 
        
        -- Wait for last bits to be send
        wait for CLK_PERIODE * 4  * MCLK_PRESCALER;
        
        -- Reset device
        Reset <= '1';
        
        -- Wait for stimulus end
        wait for 100 ms; 
    end process;

end I2S_Transmitter_TB_Arch;